.TH risc l "October 30, 2015"
.SH NAME
.B risc
\- Recoding Infrastructure for SystemC (RISC) Compiler and Simulator
.SH SYNOPSIS
.B risc
[
.I options
]
.I design
[
.I options
]
.SH DESCRIPTION
.B risc
is a dedicated compiler for the SystemC language.
The purpose of
.B risc
is to parse, analyze, instrument, and compile a SystemC source program
into an executable program for out-of-order parallel simulation.
.B risc
is a frontend source-to-source compiler for SystemC built on top of
the ROSE compiler infrastructure with GNU C++ as the backend target compiler.
As such,
.B risc
relies on and supports also most of the ROSE and GNU compiler options.

Using the command syntax shown in the synopsis above, the specified
.I design
is compiled. By default,
.B risc
reads the SystemC source file, performs preprocessing and
builds an internal representation (abstract syntax tree)
and a Segment Graph (SG) of the model. Next, static conflict analysis
is performed and the design model is instrumented
for Out-of-Order Parallel Discrete Event Simulation (OoO PDES). Finally,
instrumented C++ code is generated, compiled, and linked
into an executable file that can be run for fast parallel simulation.

On successful completion, the exit value 0 is returned.
In case of errors during processing, an error code with a
brief diagnostic message is written to
the standard error stream and the compilation is aborted
with an exit value greater than zero (i.e. 10).

For preprocessing and C++ compilation,
.B risc
relies on the availability of an external C++ compiler which
is used automatically in the background. By default, the GNU C++ compiler
.B g++
is used.
.SH ARGUMENTS
.TP
.I design
specifies the file name of the input SystemC design model;
by default, the base name of
.I design
is used as base name for all intermediate and output files;
.SH OPTIONS
.TP
.I \-h | \-\-help
print the compiler version and a brief usage information message
to standard output and quit;
.TP
.I \-v | \-\-verbose
increment the verbosity level so that all tasks performed are
logged to standard error (default: be silent);
at level 1, high-level messages about the tasks performed are displayed;
at level 2, additional details such as input and output file names are listed;
at level 3, very detailed information about each executed task is printed;
.TP
.I \-w | \-\-warnings
increment the warning level so that compiler warning messages
are enabled (default: warnings are disabled);
four levels are supported ranging
from only important warnings (level 1)
to pedantic warnings (level 4);
for most cases, warning level 2 is recommended (\-w \-w);
.TP
.I \-g | \-g level
add a symbol table suitable for debugging (e.g. gdb) to the generated
simulation executable (default: no debugging symbols);
.TP
.I \-O | \-O level
optimize the generated simulation executable
for higher execution speed and/or less memory usage (default: no optimization);
.TP
.I \-Idir
add the specified
.I dir
to the include path (extend the list of directories to be searched
for including source files);
include directories are searched in the order of their specification;
the standard include path ($SYSTEMC_LW_HOME/include
or $SYSTEMC_OOP_HOME/include) is
automatically appended to this list;
by default, only the standard include directories are searched;
.TP
.I \-Ldir
add the specified
.I dir
to the library path (extend the list of directories to be searched
for linker libraries);
the library path is searched in the specified order;
the standard library path ($SYSTEMC_OOP_HOME/lib) is
automatically appended to this list;
by default, only the standard library path is searched;
.TP
.I \-llib
add the specified
.I lib
to the list of libraries for the linker
so that the executable is linked against
.I lib;
libraries are linked in the specified order;
the standard libraries (i.e. -lsystemc) are
automatically appended to this list;
by default, only standard libraries are used;
.TP
.I \-c
perform only the preprocessing, analysis, instrumentation,
and compilation tasks; skip the final linking stage
so that only an object file is created (default: perform all tasks
including linking);
.TP
.I \-o output file
specify the name of the final output file explicitly (default: a.out);
.TP
.I \-rose:option
pass this option through to the underlying ROSE compiler (default: none);
.TP
.I \-GNU option
pass this option through to the underlying GNU compiler (default: none);
.SH ENVIRONMENT
.TP
.I RISC
is used to determine the installation directory of the RISC compiler
and simulator where the RISC system components are located.
.TP
.I SYSTEMC_LW_HOME
is used to find the RISC light-weight SystemC header files (in
directory $SYSTEMC_LW_HOME/include).
.TP
.I SYSTEMC_OOP_HOME
is used to find the RISC OoO PDES SystemC header files (in
directory $SYSTEMC_OOP_HOME/include)
and the RISC OoO PDES SystemC library (in
directory $SYSTEMC_OOP_HOME/lib).
.SH VERSION
The RISC compiler and simulator is alpha release version 0.2.1.
.SH AUTHORS
Tim Schmidt <schmidtt@uci.edu>,
Guantao Liu <guantaol@uci.edu>, and
Rainer Doemer <doemer@uci.edu>.
.SH COPYRIGHT
(c) 2015 CECS, University of California, Irvine
.SH LICENSE
Open source BSD license terms apply.
.SH BUGS, LIMITATIONS
Probably many, since this is an alpha release
of a proof-of-concept prototype implementation.
