#include "module_traverser.h"

std::vector<SgClassDefinition*> risc::traverser::find_all_modules()
{
  Rose_STL_Container<SgNode*> class_defintions =
    NodeQuery::querySubTree (get_project(),V_SgClassDefinition);

  std::vector<SgClassDefinition*> modules;

  for(Rose_STL_Container<SgNode*>::iterator
      iter  = class_defintions.begin();
      iter != class_defintions.end();
      iter++) {

    SgClassDefinition *cd = isSgClassDefinition(*iter);

    //Is the definition located in the SystemC library include path?
    //Or, is the definition in another external library?
    //If yes we don't need this definition

    bool external_lib_definition = false;
    bool sysc_lib_definition = false;

    // We compare the path of the node with the path of the given library path
    for(SgStringList::iterator
        iter  = get_project()->get_includeDirectorySpecifierList().begin();
        iter != get_project()->get_includeDirectorySpecifierList().end();
        iter++){

      if(cd->get_file_info()->get_filenameString().find((*iter).substr(2))
          != std::string::npos) {

        external_lib_definition   = true;
      }
    }

    // We have to check, if the class is defined in the namespace 'sc_core'
    // or 'sc_dt'.
    if((cd->get_qualified_name().getString().substr(0,7).
          compare("::sc_dt") == 0) ||
        (cd->get_qualified_name().getString().substr(0,9).
          compare("::sc_core") == 0)) {

        sysc_lib_definition   = true;
    }

    // In this case it is a SystemC library node
    if(external_lib_definition || sysc_lib_definition)
      continue;

    if(SageInterface::get_name(cd).compare("adpcm")==0) {
      std::cout << std::endl;
    }

    // We have to check if the class is derived from sc_module
    // If yes, we found a module in the user design space
    if(risc::tools::is_derived_from_sc_module(cd)) {

      modules.push_back(cd);
    }
  }

  return modules;
}

std::vector<SgClassDefinition*> risc::traverser::find_all_channels()
{
  Rose_STL_Container<SgNode*> class_defintions =
    NodeQuery::querySubTree (get_project(),V_SgClassDefinition);

  std::vector<SgClassDefinition*> channels;

  for(Rose_STL_Container<SgNode*>::iterator
      iter  = class_defintions.begin();
      iter != class_defintions.end();
      iter++) {

    SgClassDefinition *cd = isSgClassDefinition(*iter);

    //Is the definition located in the SystemC library include path?
    //Or, is the definition in another external library?
    //If yes we don't need this definition

    bool external_lib_definition = false;
    bool sysc_lib_definition = false;

    // We compare the path of the node with the path of the given library path
    for(SgStringList::iterator
        iter  = get_project()->get_includeDirectorySpecifierList().begin();
        iter != get_project()->get_includeDirectorySpecifierList().end();
        iter++){

      if(cd->get_file_info()->get_filenameString().find((*iter).substr(2))
          != std::string::npos) {

        external_lib_definition   = true;
      }
    }

    // We have to check, if the class is defined in the namespace 'sc_core'
    // or 'sc_dt'.
    if((cd->get_qualified_name().getString().substr(0,7).
          compare("::sc_dt") == 0) ||
        (cd->get_qualified_name().getString().substr(0,9).
          compare("::sc_core") == 0)) {

        sysc_lib_definition   = true;
    }

    // In this case it is a SystemC library node
    if(external_lib_definition || sysc_lib_definition)
      continue;

    if(SageInterface::get_name(cd).compare("adpcm")==0) {
      std::cout << std::endl;
    }

    // We have to check if the class is derived from sc_module
    // If yes, we found a module in the user design space
    if(risc::tools::is_derived_from_sc_channel(cd)) {

      channels.push_back(cd);
    }
  }

  return channels;
}

/* ex: set softtabstop=2 tabstop=2 shiftwidth=2 expandtab: */
