#ifndef INSTANCE_TREE_H_INCLUDED_
#define INSTANCE_TREE_H_INCLUDED_

#include "rose.h"

#include <vector>
#include <list>
#include <utility>

#include "variable.h"
#include "class.h"
#include "object.h"

namespace risc {

class InstanceTree {

public:
  InstanceTree();

  InstanceTree(const InstanceTree &other);

  typedef std::pair<Class*, Object*> Element;
  typedef std::list<Element> Path;

  /**
   * \function get_real_mapped_variable
   * \brief    This function returns mapped variables for a reference.
   *           If the mapped variable is a global variable, the first scope
   *           pointer will be NULL
   */
  InstanceTree get_real_mapped_variable();

  /**
   * \function find_all_instances_of
   * \brief    This function returns all variables
   */
  std::list<InstanceTree> find_all_instances_of(SgVariableDefinition *var_def);

  /**
   * \function get_parent
   * \brief    This function returns the parent variable of variable
   */
  InstanceTree get_parent();

  /**
   * \function print_instance_tree
   * \brief    Prints the instance tree
   */
  void print_instance_tree();

  /**
   * \function get_scope
   * \brief    returns the scope of the object
   */
  Class* get_scope();

  /**
   * \function get_object
   * \brief    returns the object
   */
  Object* get_object();

  /**
   * \function get_object_symbol
   * \brief    returns the symbol of the object
   */
  SgVariableSymbol* get_object_symbol();

  /**
   * \function get_all_ports
   * \brief    returns all ports
   */
  void get_all_ports(std::list<InstanceTree>& occurence);

  /**
   * \function get_all_events
   * \brief    returns all events
   */
  void get_all_events(std::list<InstanceTree>& occurence);

  /**
   * \function pop_back
   * \brief    Deletes the last element in the instance tree
   */
  Element pop_back();

  /**
   * \function push_back
   * \brief    Appends element ohs at the end of the list
   */
  void push_back(Element ohs);


  Path path_;

  bool operator==(const InstanceTree &ohs);

private:
  /**
   * \function recursive_tree_walk
   * \brief    Iterates recursively over the tree and collects all matching
   *           objects
   */
  void recursive_tree_walk(std::list<InstanceTree>& occurence,
           SgVariableDefinition *var_def);

  /**
   * \function get_mapped_reference
   * \brief    This function determines the mapped variables symbol in the
   *           parent class.
   *           Only for internal use
   */
  SgVariableSymbol* get_mapped_reference(SgVariableSymbol* reference,
    SgClassDefinition *base_class_def,
    SgClassDefinition *parent_class_def,
    SgVariableDefinition *parent_member);
};

} // end of namespace risc

#endif /* INSTANCE_TREE_H_INCLUDED_ */

/* ex: set softtabstop=2 tabstop=2 shiftwidth=2 expandtab: */
