#ifndef AST_ATTRIBUTE_COLOR_H_INCLUDED_
#define AST_ATTRIBUTE_COLOR_H_INCLUDED_

#include "rose.h"

namespace risc
{

namespace tools
{

/**
 * This class should be used to attribute ast nodes with colors
 */
class AstAttributeColor: public AstAttribute {
public:

  // http://en.wikipedia.org/wiki/List_of_8-bit_computer_hardware_palettes
  enum Color {
    BLACK = 0,
    WHITE = 1,
    RED = 2,
    CYAN = 3,
    PURPLE = 4,
    GREEN = 5,
    BLUE = 6,
    YELLOW = 7,
    NO_COLOR = 255};

  // This are 'special' colors for the function call hierarchy
  enum CallHierarchy {
    IN_CALL_HIERARCHY = 100,
    NOT_IN_CALL_HIERARCHY = 101};

  Color color_;
  CallHierarchy call_hierarchy_;

  /**
   *  \brief default constructor where the color is 'NO_COLOR'
   */
  AstAttributeColor();

  /**
   *  \brief constructor which overtakes a user given color
   */
  AstAttributeColor(Color color);
};

}; // end namespace tools

}; // end namespace risc

#endif /* AST_ATTRIBUTE_COLOR_H_INCLUDED_ */

/* ex: set softtabstop=2 tabstop=2 shiftwidth=2 expandtab: */
