#include "variable_helper.h"

bool
risc::tools::is_reference_variable(SgVariableSymbol *var_symbol)
{
  return isSgReferenceType(var_symbol->get_type());
}

bool
risc::tools::is_global_variable(SgVariableSymbol *var_symbol)
{
  if(SageInterface::getGlobalScope(var_symbol)->symbol_exists(var_symbol)) {

    return true;
  } else {
    return false;
  }
}

bool
risc::tools::is_local_variable(SgVariableSymbol *var_symbol)
{
  SgScopeStatement *scope =
    SageInterface::getEnclosingScope(var_symbol);

  SgBasicBlock *bb = isSgBasicBlock(scope);
  if(bb) {

    if(bb->symbol_exists(var_symbol)) {

      return true;
    } else {
      return false;
    }

  } else {
    return false;
  }
}

bool
risc::tools::is_member_variable(SgVariableSymbol *var_symbol)
{
  SgClassDefinition *class_def =
    SageInterface::getEnclosingClassDefinition(var_symbol);

  if(class_def == NULL) {
    return false;
  }

  if(class_def->symbol_exists(var_symbol)) {

    return true;
  } else {
    return false;
  }
}

bool
risc::tools::is_static_variable(SgVariableSymbol *var_symbol)
{
  SgInitializedName *init_name = var_symbol->get_declaration();

  SgVariableDefinition *var_def
    = isSgVariableDefinition(init_name->get_declptr());

  if(var_def) {
    return SageInterface::isStatic(var_def->get_declaration());
  } else {

    return init_name->get_storageModifier().isStatic();
  }
}

bool
risc::tools::is_namespace_variable(SgVariableSymbol *var_symbol)
{
  SgScopeStatement *scope
    = SageInterface::getEnclosingScope(var_symbol);

  SgNamespaceDefinitionStatement *namespace_def
    = isSgNamespaceDefinitionStatement(scope);

  if(namespace_def) {

    if(namespace_def->symbol_exists(var_symbol)) {

      return true;
    } else {
      return false;
    }

  } else {
    return false;
  }
}

bool
risc::tools::is_function_parameter_variable(SgVariableSymbol *var_symbol)
{
  SgFunctionDefinition *function_def
    = SageInterface::getEnclosingFunctionDefinition(var_symbol);

  if(function_def) {
    if(function_def->symbol_exists(var_symbol)) {

      return true;
    } else {
      return false;
    }
  }

  return false;
}

bool
risc::tools::is_member_of(SgVariableSymbol *is, SgSymbol *of)
{
  SgClassDeclaration *class_decl = NULL;

  if(isSgVariableSymbol(of)) {

    SgInitializedName *name = isSgVariableSymbol(of)->get_declaration();

    class_decl = isSgClassDeclaration(name->get_declaration());
    if(!class_decl) {
      return false;
    }

  } else if(isSgClassSymbol(of)) {
    class_decl = isSgClassSymbol(of)->get_declaration();

  } else {
    return false;
  }

  if(class_decl) {
    SgClassDefinition *class_def = class_decl->get_definition();

    if(class_def->symbol_exists(is)) {
      return true;
    } else {

      SgBaseClassPtrList &base_classes = class_def->get_inheritances();

      // iterate all direct base classes
      for(SgBaseClassPtrList::iterator
          base_class_iter  = base_classes.begin();
          base_class_iter != base_classes.end();
          base_class_iter++) {

        SgSymbol *symbol = (*base_class_iter)->get_base_class()->
                             search_for_symbol_from_symbol_table();


        if(is_member_of(is, symbol)) {
          return true;
        }
      }

      return false;
    }
  }
  return false;
}

/* ex: set softtabstop=2 tabstop=2 shiftwidth=2 expandtab: */
